/*
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 * 
 * Contact Information:
 * Swarco Traffic Systems GmbH
 * Entwicklung
 * Adolf-Dambach-Strae
 * D-76571 Gaggenau
 * Phone: +49 (0)7225-64-0
 * Fax: +49 (0)7225-64-300
 * mailto: office.sts@swarco.de
 */

package de.bsvrz.ste.stealgsba.algorithmen.lib;

import de.bsvrz.dav.daf.main.ClientDavConnection;
import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.config.ConfigurationObject;
import de.bsvrz.dav.daf.main.config.DataModel;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.sys.funclib.debug.Debug;

/**
 * Klasse implementiert einen realen Anzeigequerschnitt vom Typ 'typ.anzeigeQuerschnitt'.<p>
 *
 * @version $Revision: 1.2 $ / $Date: 2012/07/23 06:02:36 $ / ($Author: Asal $)
 *
 * @author Swarco Traffic Systems GmbH
 * @author Rolf Asal
 *
 */
public class AnzeigeQuerschnitt
{
	/**
   * Stellt Methoden fr die Debugausgabe zur Vefgung.
   */
	private static final Debug m_debug = Debug.getLogger();
	/**
   * Schnittstellen, die die logische Verbindung zum Datenverteiler reprsentiert. 
   */
	private ClientDavConnection m_connection;
	/**
	 * Schnittstelle zum Zugriff auf Objekte und Eigenschaften eines Datenmodells.
	 */
	private DataModel m_dataModel;
	/**
	 * Systemobjekt des Anzeigequerschnitt.
	 */
	private SystemObject m_anzeigeQuerschnitt;
	/**
	 * Offset des Anzeigequerschnitt bezogen auf das Straensegment in Zentimeter.
	 */
	private long m_offset = -1;
	/**
	 * Straensegment auf dem der Anzeigenquerschnitt steht.
	 */
	private SystemObject m_strassenSegment = null;
	
	/**
	 * Konfigurationsdaten der Attributgruppe 'atg.punktLiegtAufLinienObjekt'.
	 */
	private PunktLiegtAufLinienObjekt m_punktLiegtAufLinienObjekt;
	
	/**
	 * Konstruktor mit bernahme der Parameter.
	 * 
	 * @param connection Aktuelle Verbindung zum Datenverteiler.
	 * @param anzeigeQuerschnitt Systemobjekt Anzeigequerschnitt.
	 */
	public AnzeigeQuerschnitt(ClientDavConnection connection, SystemObject anzeigeQuerschnitt)
	{
		m_connection = connection;
		m_dataModel = m_connection.getDataModel();
		
		m_anzeigeQuerschnitt = anzeigeQuerschnitt;
		bestimmeKonfigurationsDaten(anzeigeQuerschnitt);
	}
	
	/**
	 * Diese Methode bestimmt die Konfigurationsdaten des SBA Anzeigequerschnitts
	 * und legt diese in lokale Variablen ab.
	 * 
	 * @param anzeigeQuerschnitt Objekt Anzeigequerschnitt.
	 */
	private void bestimmeKonfigurationsDaten(SystemObject anzeigeQuerschnitt)
	{
		if (anzeigeQuerschnitt != null)
		{			
			ConfigurationObject co = (ConfigurationObject)anzeigeQuerschnitt;
			if (co != null)
			{
				m_punktLiegtAufLinienObjekt = new PunktLiegtAufLinienObjekt();
				
				Data data = co.getConfigurationData(m_dataModel.getAttributeGroup("atg.punktLiegtAufLinienObjekt"));
				if (data != null)
				{
					m_offset = data.getUnscaledValue("Offset").longValue();
					m_strassenSegment = data.getReferenceValue("LinienReferenz").getSystemObject();
					
					m_punktLiegtAufLinienObjekt.setLinienReferenz(m_strassenSegment);
					m_punktLiegtAufLinienObjekt.setOffset(m_offset);
				}
				else
				{
					m_punktLiegtAufLinienObjekt.setLinienReferenz(m_dataModel.getObject("iss.n.11571.n.11569.0"));
					m_punktLiegtAufLinienObjekt.setOffset(10000);					
				}				
			}
		}
		
		return;
	}

	/**
	 * Gibt die Pid des Anzeigequerschnitts zurck.
	 * @return Pid des Anzeigequerschnitts.
	 */
	public String getPid()
	{
		if (m_anzeigeQuerschnitt != null)
		{
			return m_anzeigeQuerschnitt.getPid();
		}
		return null;
	}

	/**
	 * Gibt das Objekt des realen Anzeigequerschnitts zurck.
	 * @return Objekt des realen Anzeigequerschnitts.
	 */
	public SystemObject getObjektAnzeigeQuerschnitt()
	{
		return m_anzeigeQuerschnitt;
	}

	/**
	 * Gibt den Offset des Anzeigequerschnitt bezogen auf das Straensegment in Zentimeter zurck.
	 * @return Offset des Anzeigequerschnitt bezogen auf das Straensegment in Zentimeter.
	 */
	public long getOffset()
	{
		return m_offset;
	}

	/**
	 * Gibt das Straensegment auf dem der Anzeigenquerschnitt steht zurck.
	 * @return Straensegment auf dem der Anzeigenquerschnitt steht.
	 */
	public SystemObject getStrassenSegment()
	{
		return m_strassenSegment;
	}

	/**
	 * Gibt die Klasse PunktLiegtAufLinienObjekt zurck.
	 * @return Klasse PunktLiegtAufLinienObjekt.
	 */
	public PunktLiegtAufLinienObjekt getPunktLiegtAufLinienObjekt()
	{
		return m_punktLiegtAufLinienObjekt;
	}
}
