/*
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 * 
 * Contact Information:
 * Swarco Traffic Systems GmbH
 * Entwicklung
 * Adolf-Dambach-Strae
 * D-76571 Gaggenau
 * Phone: +49 (0)7225-64-0
 * Fax: +49 (0)7225-64-300
 * mailto: office.sts@swarco.de
 */

package de.bsvrz.ste.stealgsba.algorithmen.lib;

import java.util.Iterator;
import java.util.Vector;

import de.bsvrz.dav.daf.main.ClientDavConnection;
import de.bsvrz.dav.daf.main.ClientReceiverInterface;
import de.bsvrz.dav.daf.main.DataDescription;
import de.bsvrz.dav.daf.main.DataNotSubscribedException;
import de.bsvrz.dav.daf.main.ReceiveOptions;
import de.bsvrz.dav.daf.main.ReceiverRole;
import de.bsvrz.dav.daf.main.ResultData;
import de.bsvrz.dav.daf.main.config.Aspect;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.dav.daf.main.config.DataModel;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.ste.stealgsba.algorithmusrahmen.interfaces.IUpdateNeuedaten;
import de.bsvrz.sys.funclib.debug.Debug;

/**
 * Diese Klasse implementiert ein Objekt vom Typ 'typ.deLve'.
 *
 * @version $Revision: 1.3 $ / $Date: 2014/03/27 10:19:17 $ / ($Author: Asal $)
 *
 * @author Swarco Traffic Systems GmbH
 * @author Rolf Asal
 *
 */
public class DeLve implements ClientReceiverInterface
{
	/**
   * Stellt Methoden fr die Debugausgabe zur Vefgung.
   */
	private Debug m_debug = Debug.getLogger();
	/**
	 * Aktuelle Verbindung zum Datenverteiler.
	 */
	private ClientDavConnection m_connection;
	/**
	 * Zugriff auf das aktuelle Datenmodell.
	 */
	private DataModel m_dataModel;
	/**
	 * Systemobjekt des Fahrstreifen.
	 */
	private SystemObject m_objekt;	
	/**
	 * Attributgruppe fr den Empfang der Kanalsteuerung.
	 */
	private AttributeGroup m_atgTlsGloKanalSteuerung;
	/**
	 * Aspekt fr den Empfang der Kanalsteuerung.
	 */
	private Aspect m_aspParameterSoll;	
	/**
	 * Status der Kanalsteuerung (aktiv/passiv).
	 */
	private String m_deKanalStatus;
	
	/**
	 * Listener neue Daten erhalten.
	 */
	private Vector<IUpdateNeuedaten> m_listener = new Vector<IUpdateNeuedaten>();

	/**
	 * Konstruktor mit bernahme der Parameter.
	 * 
	 * @param connection Aktuelle Verbindung zum Datenverteiler.
	 * @param objekt Objekt
	 */
	public DeLve(ClientDavConnection connection, SystemObject objekt)
	{
		m_connection = connection;
		m_dataModel = m_connection.getDataModel();
		
		m_objekt = objekt;
		
		m_atgTlsGloKanalSteuerung = m_dataModel.getAttributeGroup("atg.tlsGloKanalSteuerung");
		m_aspParameterSoll = m_dataModel.getAspect("asp.parameterSoll");
	}

	/**
	 * Anmelden der entsprechenden Daten zum Empfang am Datenverteiler.
	 */
	public void datenDavAnmelden()
	{
		empfaengerAnmeldenObjekt(this, m_objekt, m_atgTlsGloKanalSteuerung, m_aspParameterSoll);
	}

	/**
   * Fr ein Objekt Attributgruppe/Aspekt Kombination am Datenverteiler anmelden.
   * 
   * @param instanz Instanz.
   * @param objekt Objekt.
   * @param atg Attributgruppe.
   * @param asp Aspekt.
   * @return true = Anmeldung erfolgreich, false = nicht erfolgreich.
   */
  private boolean empfaengerAnmeldenObjekt(ClientReceiverInterface instanz, SystemObject objekt, AttributeGroup atg, Aspect asp)
  {
    DataDescription dd;
    boolean ok = false;

    m_debug.fine(("DatenAnmeldenObjekt " + atg.getPid() + " " + asp.getPid() + " fr " + objekt));
    
    try
    {  
    	dd = new DataDescription(atg, asp);

    	m_connection.subscribeReceiver(instanz, objekt, dd, ReceiveOptions.normal(), ReceiverRole.receiver());
        
    	ok = true;
    } 
    catch (DataNotSubscribedException e)
    {
      m_debug.error(("Daten nicht zum Empfangen angemeldet " + e.getMessage()));
    } 
    catch (IllegalArgumentException e)
    {
      m_debug.error(("Daten nicht zum Empfangen angemeldet " + e.getMessage()));
    } 
    
    return (ok);
  }

	/* (non-Javadoc)
	 * @see de.bsvrz.dav.daf.main.ClientReceiverInterface#update(de.bsvrz.dav.daf.main.ResultData[])
	 */
	@Override
	public void update(ResultData[] results)
	{
		for (ResultData resultData : results)
		{
			if (resultData.getDataDescription().getAttributeGroup().equals(m_atgTlsGloKanalSteuerung))
			{
				if (resultData.hasData())
				{
					m_deKanalStatus = resultData.getData().getScaledValue("DEKanalStatus").getValueText();
				
					fireEvent();
				}
			}
		}
		
	}

	/**
	 * Gibt zurck, ob die DE logisch passiviert ist.
	 * @return true wenn DE passiv, ansonsten false. 
	 */
	public boolean isPassiviert()
	{
		if (m_deKanalStatus != null)
		{
			return m_deKanalStatus.equals("passiv");
		}
		
		return false;
	}

	/**
	 * Methode fgt den bergebenen Listener zur aktuellen Liste hinzu.
	 * 
	 * @param listener IUpdateAnzeige.
	 */
	public void addListener(IUpdateNeuedaten listener)
	{		
		if (!m_listener.contains(listener))
		{
			m_listener.add(listener);
		}
	}

	/**
	 * Methode lscht einen angemeldeten Listener.
	 * 
	 * @param listener IUpdateAnzeige.
	 */
	public void removeListener(IUpdateNeuedaten listener)
	{
		if (m_listener.contains(listener))
		{
			m_listener.remove(listener);
		}
	}
	
	/**
	 * Methode informiert alle angemeldeten Listener ber neue Daten.
	 */
	protected void fireEvent()
	{
		for (Iterator<IUpdateNeuedaten> iterator = m_listener.iterator(); iterator.hasNext();)
		{
			IUpdateNeuedaten iUpdNd = (IUpdateNeuedaten)iterator.next();
			
//			System.out.println("IUpdateNeueDaten " + " OBJ " + m_objekt.getPid() + " " + iUpdNd);
			
			iUpdNd.neueDatenErhalten();
		}
	}

}
